/**
 * @file ulcollinsxhtmlrenderer.h
 * @brief The interface for the <code>ULCollinsWebRenderer</code> class.
 * @author Jeff Ondich
 */

#ifndef ULCOLLINSWEBRENDERER_H
#define ULCOLLINSWEBRENDERER_H

#include "ulxhtmlrenderer.h"

/**
 * @class ULCollinsWebRendererState
 * @brief This descendant of <code>ULRendererState</code> maintains
 * the state required by ULCollinsWebRendererState::toString.
 */
class ULCollinsWebRendererState : public ULXHTMLRendererState
{
private:
    bool mNeedsComma;
    bool mInSense;
	bool mLastDefTextWasEmpty;
    uint32 mNDefinitionNumbers;
	ulstring mSubentryPhrase;

public:
    ULCollinsWebRendererState();
    ULCollinsWebRendererState( const ULCollinsWebRendererState& state );
    ULCollinsWebRendererState( const ULDMIterator *uldmItPtr );
    virtual ~ULCollinsWebRendererState();

    virtual void clear();
    virtual ULCollinsWebRendererState& operator=( const ULCollinsWebRendererState& state );

	void setSubentryPhrase(ulstring s);
	bool hasSubentryPhrase() const;
	ulstring getSubentryPhrase() const;
    void setNeedsComma( bool b );
    bool hasNeedsComma() const;
    void setInSense( bool b );
    bool hasInSense() const;
    void setNDefinitionNumbers( uint32 n );
	bool hasLastDefTextWasEmpty() const;
	void setLastDefTextWasEmpty( bool b );
    uint32 getNDefinitionNumbers() const;
};


/**
 * @class ULCollinsWebRenderer
 * @brief The child of <code>ULXHTMLRenderer</code> that is used
 * to render Collins ULD data in XHTML form suitable for display
 * on the web or in desktop and handheld applications.
 */
class ULCollinsWebRenderer : public ULXHTMLRenderer
{
public:
    ULCollinsWebRenderer();
    ULCollinsWebRenderer( const ULCollinsWebRenderer& renderer );
    virtual ~ULCollinsWebRenderer();

    virtual void clear();
    virtual ULCollinsWebRenderer& operator=( const ULCollinsWebRenderer& renderer );

protected:
    static bool isLabel( uint32 featureID );
    static bool endsWithAlpha( const ulstring& s );

    // Overrides.
    virtual ULRendererState *createRendererState( const ULDMIterator& uldmIt );
    virtual void initFeatures();
    virtual void startTerm( ULRendererState *state, ulstring& resultString );
    virtual void endTerm( ULRendererState *state, ulstring& resultString );
    virtual void processTermText( ULRendererState *state, ulstring& resultString );
    virtual void processTermPOSP( ULRendererState *state, ulstring& resultString );

    virtual void startDefinitions( ULRendererState *state, ulstring& resultString );
    virtual void endDefinitions( ULRendererState *state, ulstring& resultString );
    virtual void startDefinition( ULRendererState *state, ulstring& resultString );
    virtual void endDefinition( ULRendererState *state, ulstring& resultString );
    virtual void processDefinitionText( ULRendererState *state, ulstring& resultString );
    virtual void processDefinitionPOSP( ULRendererState *state, ulstring& resultString );

    virtual void processFeature( uint32 featureID,
                                 const ulstring& featureValue,
                                 ULFeatureContext featureContext,
                                 ULRendererState *state,
                                 ulstring& resultString );

    virtual void prepareFeatureText( uint32 featureID,
                                     ULRendererState *state,
                                     ulstring& text );

    virtual void prepareLinkedFeatureText( uint32 featureID,
                                           ULRendererState *state,
                                           bool linkFullTextOnly,
                                           ulstring& text );

    virtual ulstring getLink( uint32 featureID,
                              ULRendererState *state,
                              const ulstring& text );

    void fixInflection( ulstring& inflectionText );
};

#endif

