/**
 * @file ulxhtmlrenderer.h
 * @brief The interface for the <code>ULXHTMLRenderer</code> class.
 * @author Jeff Ondich
 */

#ifndef ULXHTMLRENDERER_H
#define ULXHTMLRENDERER_H

#include "ulsimplerenderer.h"

/**
 * @class ULXHTMLRendererState
 * @brief This child of <code>ULRendererState</code> maintains
 * the state required by ULXHTMLRendererState::toString.
 */
class ULXHTMLRendererState : public ULRendererState
{
private:
    bool mAddWordLinks;

public:
    ULXHTMLRendererState();
    ULXHTMLRendererState( const ULXHTMLRendererState& state );
    ULXHTMLRendererState( const ULDMIterator *uldmItPtr );
    virtual ~ULXHTMLRendererState();

    virtual void clear();
    virtual ULXHTMLRendererState& operator=( const ULXHTMLRendererState& state );

    virtual void setAddWordLinks( bool b );
    virtual bool hasAddWordLinks() const;
};


/**
 * @class ULXHTMLRenderer
 * @brief This child of <code>ULSimpleRenderer</code> provides
 * utility functions for child classes charged with rendering XHTML
 * in various contexts.
 */
class ULXHTMLRenderer : public ULSimpleRenderer
{
public:
    ULXHTMLRenderer();
    ULXHTMLRenderer( const ULXHTMLRenderer& renderer );
    virtual ~ULXHTMLRenderer();

    virtual void clear();
    virtual ULXHTMLRenderer& operator=( const ULXHTMLRenderer& options );

protected:
    // Overrides
    virtual ULRendererState *createRendererState( const ULDMIterator& uldmIt );

    // Overridable utilities.
    virtual void prepareFeatureText( uint32 featureID,
                                     ULRendererState *state,
                                     ulstring& text );

    virtual void prepareLinkedFeatureText( uint32 featureID,
                                           ULRendererState *state,
                                           bool linkFullTextOnly,
                                           ulstring& text );

    virtual void insertLinks( uint32 featureID,
                              ULRendererState *state,
                              ulstring& text );

    virtual ulstring getLink( uint32 featureID,
                              ULRendererState *state,
                              const ulstring& text );

    // Other utilities.
    void fixXHTMLEntitites( ulstring& s );
    void fixQuotationMarks( ulstring& s );
    void fixEscapeSequences( ulstring& s );
};

#endif

