/**
 * @file ulstemmer.h
 * @brief The interface for the <code>ULStemmer</code> class.
 * @author Jeff Ondich
 */

#ifndef ULSTEMMER_H
#define ULSTEMMER_H

#include "uldmanager.h"
#include "ulconjugator.h"
#include "ultokenizer.h"

class ULStemmer
{
private:
    ULDManager      *mULDManager;
    ULConjugator    *mConjugator;
    ULTokenizer     *mTokenizer;

    void            addToStemList( ULList< ULPair<ULStem,ULDMIterator> >& stemList,
                                    const ulstring& searchString,
                                    const ULPair<ULStem,ULDMIterator>& stemPair,
                                    ULLanguage stemmingLanguage );

public:
    static ULStemmer *create();

                    ULStemmer();
                    ~ULStemmer();

    bool            init();

    void            setULDManager( ULDManager *manager );
    void            setConjugator( ULConjugator *conjugator );
    void            setTokenizer( ULTokenizer *tokenizer );

    void            getStems( ULList< ULPair<ULStem,ULDMIterator> >& stemList,
                              const ulstring& searchString,
                              ULLanguage stemmingLanguage,
                              ULLanguage translationLanguage,
                              uint32 product,
                              int verbMatchingStyle=kULCFirstMatchPerVerb );

    static void     stemListToHeadTermList( const ULList< ULPair<ULStem,ULDMIterator> >& stemList, ULList<ULDMIterator>& headTermList );
};

#endif
