/**
 * @file ulstringpool.h
 * @brief The interface for the <code>ULStringPool</code> class.
 * @author Jeff Ondich
 */

#ifndef ULSTRINGPOOL_H
#define ULSTRINGPOOL_H

#include "ulstring.h"

/**
 * @class ULStringPool
 * @brief A class used to hold a collection of localized strings used
 * in the rendering of ULD data.
 */

class UL_API ULStringPool
{
public:
    ULStringPool();
    ULStringPool( ULLanguage lang, uint32 nStrings );
    ~ULStringPool();

    void clear();
    void reset( uint32 nStrings );
    uint32 getPoolSize() const;
    ULStringPool& operator=( const ULStringPool& pool );
    ULLanguage getLanguage() const;
    void setLanguage( ULLanguage lang );
    void setString( int stringId, const ulstring& s );
    bool getString( int stringId, ulstring& s ) const;

private:
    uint32 mNStrings;
    ulstring **mStrings;
    ULLanguage mLanguage;
}; 

#endif

