////////////////////////////////////////////////////////////////////////////////
//
// ulutilities.h
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ULUTILITIES_H
#define ULUTILITIES_H

#include "ulstring.h"
#include "ulcontainers.h"

class UL_API ULUtilities
{
public:
    static void trim( ulstring& s );
    static void reverse( ulstring& s );
    static void replace( ulstring& s, uint32 nReplacements, const char *toReplace, const char *replacement );
    static void replaceAll( ulstring& s, const char *toReplace, const char *replacement );
    static void split( const ulstring& s, const ulstring& delimiter, ULList<ulstring>& pieces );
    static ulstring xmlEscape( const ulstring& s );
    static ulstring prepareSearchString( const ulstring& s );

    static void sortLanguages( ULLanguage& lang1, ULLanguage& lang2 );
    static ULLanguage stringToLanguage( const ulstring& str );
    static void languageToEnglishString( ULLanguage lang, ulstring& str );
    static void languageToAbbreviation( ULLanguage lang, ulstring& str );

    static uint32 stringToProduct( const ulstring& str );
    static void productToString( uint32 product, ulstring& str );

    static uint32 stringToPlatform( const ulstring& str );
    static void platformToString( uint32 platform, ulstring& str );

    static ULOutputType stringToOutputType( const ulstring& str );
    static void outputTypeToString( ULOutputType outputType, ulstring& str );

    static ULSearchType stringToSearchType( const ulstring& str );
    static void searchTypeToString( ULSearchType searchType, ulstring& str );

    // Klingon support.
    static void piqadToRomanizedKlingon( const ulstring& in, ulstring& result );
    static void romanizedKlingonToPIQAD( const ulstring& in, ulstring& result );

    // Chinese support
    static void pinyinDigitsToDiacritics( const ulstring& in, ulstring& result );
    static void pinyinDiacriticsToDigits( const ulstring& in, ulstring& result );

    // Reading and writing with buffers.
    static uint16 readInt16FromBuffer( const uint8 *buffer );
    static uint32 readInt24FromBuffer( const uint8 *buffer );
    static uint32 readInt32FromBuffer( const uint8 *buffer );

    // These return a pointer to the byte immediately following the data
    // written by the functions.
    static uint8 *writeStringToBuffer( uint8 *buffer, const ulstring& s, ulchar delimiter=ulchar('\0'), uint32 bufferSize=0 );
    static uint8 *writeInt16ToBuffer( uint8 *buffer, uint32 n );
    static uint8 *writeInt24ToBuffer( uint8 *buffer, uint32 n );
    static uint8 *writeInt32ToBuffer( uint8 *buffer, uint32 n );

    // If you need an integer containing no zero bytes, use these
    // functions to convert between the normal representation and
    // a representation we're calling "base 255".  The idea is that
    // each byte represents a "digit" between 0 (represented in binary
    // as 00000001) and 254 (represented as 11111111).  Then you combine
    // four such digits into a single base-255 number, none of whose
    // bytes is represented as 00000000.  This can be handy if you have
    // a buffer or file with null-terminated records containing integers.
    // With base-255 integers, there's no ambiguity about what a 0-byte means.
    static uint32 intToBase255( uint32 n );
    static uint16 int16ToBase255( uint16 n );
    static uint32 base255ToInt( uint32 n );
    static uint16 base255ToInt16( uint16 n );

    static void getPalmDBName( ulstring& s, ULLanguage red, ULLanguage green, uint32 version, bool additions=false );
    static void updateSearchProgress();

#if defined(UL_ANSI) && !defined(UL_NOSTL)
    static void writeInt16( ostream& out, uint32 n );
    static void writeInt24( ostream& out, uint32 n );
    static void writeInt32( ostream& out, uint32 n );

    static void readInt16( istream& in, uint16& n );
    static void readInt24( istream& in, uint32& n );
    static void readInt32( istream& in, uint32& n );

    static void makeSpaceInFile( fstream& out, int nBytesToInsert, int insertionPoint );

#endif

#ifdef UL_WINDOWSMOBILE
    static void reportLastError();

    static void writeInt16( HANDLE fileHandle, uint32 n );
    static void writeInt24( HANDLE fileHandle, uint32 n );
    static void writeInt32( HANDLE fileHandle, uint32 n );

    static void readInt16( HANDLE fileHandle, uint16& n );
    static void readInt24( HANDLE fileHandle, uint32& n );
    static void readInt32( HANDLE fileHandle, uint32& n );
#endif
};

#endif
