////////////////////////////////////////////////////////////////////////
// ulchars.h
// Character utility functions using the ulchar type.
////////////////////////////////////////////////////////////////////////

#ifndef ULCHARS_H
#define ULCHARS_H

#include "ultypes.h"

UL_API bool ul_iswordchar( ulchar ch );
UL_API bool ul_isvowel( ulchar ch );
UL_API bool ul_isseparator( ulchar ch );

UL_API ulchar ul_rank( ulchar ch, ULLanguage lang=kULNeutralLanguage );
UL_API ulchar ul_rank_swedish( ulchar ch );
UL_API ulchar ul_base( ulchar ch, ULLanguage lang=kULNeutralLanguage );
UL_API ulchar ul_base_swedish( ulchar ch );
UL_API ulchar ul_tolower( ulchar ch );
UL_API ulchar ul_toupper( ulchar ch );

UL_API bool ul_isspace( ulchar ch );
UL_API bool ul_isdigit( ulchar ch );
UL_API bool ul_isalpha( ulchar ch );
UL_API bool ul_isCJK( ulchar ch );

ulchar ul_add_accute_accent( ulchar ch );
bool ul_hasumlaut( ulchar ch );
ulchar ul_add_umlaut( ulchar ch );

// Handy character constants.
const ulchar kULCharSpace = 32;
const ulchar kULCharApostrophe = 39;
const ulchar kULCharComma = 44;
const ulchar kULCharHyphen = 45;
const ulchar kULNonbreakingSpace = 0xA0;
const ulchar kULWordJoiner = 0x2060;
const ulchar kULLeftSingleQuote = 0x2018;
const ulchar kULRightSingleQuote = 0x2019;

const ulchar kULChar_OE = 0x152;
const ulchar kULChar_oe = 0x153;
const ulchar kULChar_ae = 230;
const ulchar kULChar_cCedilla = 231;
const ulchar kULCharEsstzet = 223;
const ulchar kULChar_aAcute = 225;
const ulchar kULChar_aGrave = 224;
const ulchar kULChar_aUmlaut = 228;
const ulchar kULChar_eAcute = 233;
const ulchar kULChar_eGrave = 232;
const ulchar kULChar_eUmlaut = 235;
const ulchar kULChar_oAcute = 243;
const ulchar kULChar_oGrave = 242;
const ulchar kULChar_oUmlaut = 246;
const ulchar kULChar_uAcute = 250;
const ulchar kULChar_uGrave = 249;
const ulchar kULChar_uUmlaut = 252;

const ulchar kULCharBlackDiamond = 0x25C6;
#define ULCHAR_BLACK_DIAMOND "\xE2\x97\x86"
const ulchar kULCharWhiteDiamond = 0x25C7;
#define ULCHAR_WHITE_DIAMOND "\xE2\x97\x87"
const ulchar kULCharBlackLozenge = 0x29EB;
#define ULCHAR_BLACK_LOZENGE "\xE2\xA7\xAB"
const ulchar kULCharWhiteLozenge = 0x25CA;
#define ULCHAR_WHITE_LOZENGE "\xE2\x97\x8A"
const ulchar kULCharRightArrow = 0x2192;
#define ULCHAR_RIGHT_ARROW "\xE2\x86\x92"

// These are used in German conjugation
const ulchar kULCharSeparator = 0x000000BC; // the one-quarter character
const ulchar kULCharSimpleSeparator = 0x000000B7; // bullet
const ulchar kULCharNoSeparator = 0x0000002B; // +

// This is used in the Collins Russian data and our Russian conjugator.
const ulchar kULCharCombiningAcuteAccent = 0x00000301;

// Cyrillic letters.
const ulchar kULCharCyrillicSmallBe = 0x0431;
const ulchar kULCharCyrillicSmallVe = 0x0432;
const ulchar kULCharCyrillicSmallDe = 0x0434;
const ulchar kULCharCyrillicSmallIe = 0x0435;
const ulchar kULCharCyrillicSmallZhe = 0x0436;
const ulchar kULCharCyrillicSmallZe = 0x0437;
const ulchar kULCharCyrillicSmallI = 0x0438;
const ulchar kULCharCyrillicSmallShortI = 0x0439;
const ulchar kULCharCyrillicSmallEl = 0x043B;
const ulchar kULCharCyrillicSmallEm = 0x043C;
const ulchar kULCharCyrillicSmallPe = 0x043F;
const ulchar kULCharCyrillicSmallEr = 0x0440;
const ulchar kULCharCyrillicSmallEs = 0x0441;
const ulchar kULCharCyrillicSmallTe = 0x0442;
const ulchar kULCharCyrillicSmallU = 0x0443;
const ulchar kULCharCyrillicSmallChe = 0x0447;
const ulchar kULCharCyrillicSmallSha = 0x0448;
const ulchar kULCharCyrillicSmallShcha = 0x0449;
const ulchar kULCharCyrillicSmallSoftSign = 0x044C;
const ulchar kULCharCyrillicSmallYu = 0x044E;
const ulchar kULCharCyrillicSmallIo = 0x0451;
#endif

