//////////////////////////////////////////////////////////////////
//
//  ultypes.h
//
//  Types and defined items that are global for the entire
//  Ultralingua project.
//
//  History
//    6/20/01 (Jeff Ondich)  Started with a few handy types
//    and a central location for #defined constants.
//
//    7/26/01 Rearranged, and moved the actual definition
//    of the "essential constants" onto the compiler's command
//    line.
//
//    4/26/04 Started over for version 5 data structures.
//
//////////////////////////////////////////////////////////////////

#ifndef ULTYPES_H
#define ULTYPES_H

//////////////////////////////////////////////////////////////////
// Platform detection
//////////////////////////////////////////////////////////////////

#ifdef __APPLE__
#ifndef UL_ANSI
#define UL_ANSI
#endif
#define UL_REGEX
#endif

//////////////////////////////////////////////////////////////////
// Standard includes
//////////////////////////////////////////////////////////////////

#ifndef UL_SYMBIAN
#include <cstdlib>
#include <cctype>
#endif

#if defined(UL_ANSI)
#if !defined(UL_NOSTL)
#include <iostream>
#include <fstream>
#include <list>
#include <map>
#include <string>
using namespace std;
#if defined(UL_DEBIAN)
#include <string.h>
#endif
#else
#include <string.h>
#endif
#endif

#ifdef UL_SYMBIAN
#include <e32base.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#endif

#ifdef UL_WINDOWSMOBILE
#include <windows.h>
#endif

#ifdef UL_PALM_PDK
#include <string.h>
#endif

//////////////////////////////////////////////////////////////////
//    Essential constants
//////////////////////////////////////////////////////////////////

#if defined(DEBUG) || defined(_DEBUG)
const bool kULDebug = true;
#else
const bool kULDebug = false;
#endif

#ifdef UL_DO_INTEGRITY_CHECKS
const bool kULCheckIntegrity = true;
#else
const bool kULCheckIntegrity = false;
#endif

#ifdef UL_VERBOSE
const bool kULVerbose = true;
#else
const bool kULVerbose = false;
#endif

#ifdef UL_XMLRPC
const bool kULXMLRPC = true;
#else
const bool kULXMLRPC = false;
#endif

//////////////////////////////////////////////////////////////////
// Platform-dependent definitions of some basic types
// and macros.
//
// ULAPI uses assertions throughout.  If DEBUG is
// defined, go ahead and define assert() here as you wish.
//////////////////////////////////////////////////////////////////

#ifdef ULAPI_DLL_EXPORT
#   define UL_API __declspec(dllexport)

#elif defined( ULAPI_DLL )
#   define UL_API __declspec(dllimport)

#else
#   define UL_API
#endif


#ifdef UL_TRACE
#define    ULTrace( x )    ULUtilities::Trace( ULUtilities::Pretrace x, __FILE__, __LINE__ ) 
#else
#define ULTrace(x)
#endif



#ifdef UL_ANSI
#define     UL_ABSTRACT         =0
#define     UL_VIRTUAL          virtual
typedef     unsigned char       uchar;
typedef     unsigned char       uint8;
typedef     unsigned short      uint16;
typedef     short               int16;

#ifdef __APPLE__
typedef     unsigned int        ULUInt32;
typedef     long                int32;
#define uint32 ULUInt32

#elif defined(UL_DEBIAN) && defined(UL_64)
typedef     unsigned int        uint32;
typedef     int                 int32;
typedef     uint32              ULUInt32;

#else
typedef     unsigned long       uint32;
typedef     long                int32;
typedef     uint32              ULUInt32;
#endif

#endif


#ifdef UL_SYMBIAN
#define     UL_ABSTRACT         =0
#define     UL_VIRTUAL          virtual
typedef     TUint8              uchar;
typedef     TUint8              uint8;
typedef     TUint16             uint16;
typedef     TInt16              int16;
typedef     TUint32             uint32;
typedef     TInt32              int32;
typedef     uint32              ULUInt32;
#endif


#ifdef UL_WINDOWSMOBILE
#define     UL_ABSTRACT         =0
#define     UL_VIRTUAL          virtual
typedef     unsigned char       uchar;
typedef     unsigned char       uint8;
typedef     unsigned short      uint16;
typedef     short               int16;
typedef     unsigned long       uint32;
typedef     long                int32;
typedef     uint32              ULUInt32;
#endif


#ifdef UL_PALMOS
#include <PalmOS.h>
#define     UL_ABSTRACT
#define     UL_VIRTUAL
typedef     UInt8               uchar;
typedef     UInt8               uint8;
typedef     UInt16              uint16;
typedef     Int16               int16;
typedef     UInt32              uint32;
typedef     Int32               int32;
typedef     uint32              ULUInt32;
#endif

typedef     uint32              ulchar;
typedef     uchar *             ucharptr;


//////////////////////////////////////////////////////////////////
// Version
// 
// The 32-bit version format for .uld files is
//
//      MMMM mmmm ssss ssss PPPP PPPP pppp pppp
//
// where MMMM is the major version number, mmmm is the minor
// version number, ssssssss is the sub-minor version number,
// PPPPPPPP is the platform, and pppppppp is the product.
// The kULNoPlatform constant is used for the platform of
// all versions compiled with UL_ANSI defined.
//
// The 32-bit version format for .ulc files is
//
//      MMMM mmmm ssss ssss 0000 0000 0000 0000
//
//////////////////////////////////////////////////////////////////

#if defined(UL_ACCIO)
const uint32 kULRegKeyMajorVersionNumber = 5;
#elif defined(UL_VERSION7)
const uint32 kULRegKeyMajorVersionNumber = 7;
#else
const uint32 kULRegKeyMajorVersionNumber = 6;
#endif

const uint32 kULCurrentMajorVersionNumber = 7;
const uint32 kULMinAllowedULDVersion = 0x54000000;
const uint32 kULMaxAllowedULDVersion = 0x70FFFFFF;
const uint32 kULCurrentULDVersion = 0x70000000;
const uint32 kULMinAllowedULCVersion = 0x70000000;
const uint32 kULMaxAllowedULCVersion = 0x70FFFFFF;
const uint32 kULCurrentULCVersion = 0x70000000;

const uint32 kULVersionMask = 0xfff00000;
const uint32 kULMajorVersionMask = 0xf0000000;
const uint32 kULMinorVersionMask = 0x0f000000;
const uint32 kULSubminorVersionMask = 0x00f00000;
const uint32 kULDMagicNumber = 0xC0C0FFEE;
const uint16 kULCMagicNumber = 0xDADA;

#define UL_MAJOR_VERSION(v)     ((v >> 28) & 0x0000000f)
#define UL_MINOR_VERSION(v)     ((v >> 24) & 0x0000000f)
#define UL_SUBMINOR_VERSION(v)  ((v >> 16) & 0x000000ff)
#define UL_PLATFORM(v)          ((v >> 8) & 0x000000ff)
#define UL_PRODUCT(v)           (v & 0x000000ff)

// Products.  These are used to identify special versions of the corpora,
// and the corresponding special versions of the registration keys.  Because
// of the way the keys are formed, the number of products should not go higher
// than 99.  That was a pretty stupid decision, but we're stuck with it unless
// we want to change the key format and algorithm.
const uint32 kULUltralinguaProduct = 0;
const uint32 kULEurekaProduct = 1; // used only for keys, since there's no .uld.
const uint32 kULMedicalProduct = 2;
const uint32 kULMcGrawHillProduct = 3;
const uint32 kULExpressProduct = 4;
const uint32 kULHachetteProduct = 5; // used only for keys, since it uses the default .ulds.
const uint32 kULAccioProduct = 6;
const uint32 kULAccioIPodProduct = 7; // used only for keys
const uint32 kULCobuildProduct = 8;
const uint32 kULMidrangeProduct = 9;
const uint32 kULCollinsProduct = 10;
const uint32 kUL80KProduct = 10;
const uint32 kULCEDProduct = 11;
const uint32 kULCEDLiteProduct = 12;
const uint32 kULCEDProProduct = 13;
const uint32 kULConciseProduct = 14;
const uint32 kULCobuildAdvancedProduct = 15;
const uint32 kULCEDPlusThesaurusProduct = 16;
const uint32 kULVoxManualProduct = 17;
const uint32 kULVoxGeneralProduct = 18;
const uint32 kULVoxProduct = 19;
const uint32 kULVoxMedicalProduct = 20;
const uint32 kULNProducts = 21;

#define ULPRODUCTS_COMPATIBLE(a,b) ((a)==(b) || ((a)==kULUltralinguaProduct && (b)==kULMedicalProduct) || ((a)==kULMedicalProduct && (b)==kULUltralinguaProduct))
#define ULPRODUCT_VALID(a) ((a) >= kULUltralinguaProduct && (a) < kULNProducts)
#define ULPRODUCT_COLLINS_LITE(a) ((a) == kULCEDProduct || (a) == kULCEDLiteProduct)
#define ULPRODUCT_COLLINS(a) ((a) == kULCEDProduct || (a) == kULCEDLiteProduct || (a) == kULCollinsProduct || (a) == kULCobuildProduct || (a) == kULMidrangeProduct || (a) == kULCEDProProduct || (a) == kULConciseProduct || (a) == kULCobuildAdvancedProduct || (a) == kULCEDPlusThesaurusProduct || (a) == kUL80KProduct)
#define ULPRODUCT_CED(a) ((a) == kULCEDProduct || (a) == kULCEDLiteProduct || (a) == kULCEDProProduct || (a) == kULCEDPlusThesaurusProduct)
#define ULPRODUCT_COBUILDADVANCED(a) ((a) == kULCobuildAdvancedProduct)
#define ULPRODUCT_VOX(a) ((a) == kULVoxManualProduct || (a) == kULVoxGeneralProduct || (a) == kULVoxProduct || (a) == kULVoxMedicalProduct)

const uint32 kULDefaultProduct = kULUltralinguaProduct;

// Platforms
const uint32 kULNoPlatform = 0;
const uint32 kULNetwork = 1;
const uint32 kULMacintosh = 2;
const uint32 kULWindows = 3;
const uint32 kULPalm = 4;
const uint32 kULPocketPC = 5;
const uint32 kULSymbian = 6;
const uint32 kULSmartphone = 7;
const uint32 kULNPlatforms = 7;


//////////////////////////////////////////////////////////////////
// Language constants
//
// It's essential that these start at 1 and go in numerical
// order up to kULNLanguages.
//////////////////////////////////////////////////////////////////
enum ULLanguage
{
    kULNeutralLanguage = 0,
    kULEnglish,
    kULFrench,
    kULSpanish,
    kULGerman,
    kULItalian,
    kULPortuguese,
    kULNorwegian,
    kULLatin,
    kULEsperanto,
    kULRussian,
    kULDutch,
    kULPolish,
    kULChinese,
    kULKlingon,
    kULTurkish,
    kULHaitian,
	kULSwedish,
    kULFinnish,
    kULHungarian,
    kULKorean,
	
    kULEndLanguages,
    kULSynonyms=255
};

const uint16 kULNLanguages = kULEndLanguages - 1;


//////////////////////////////////////////////////////////////////
// Search types
//////////////////////////////////////////////////////////////////
enum ULSearchType
{
    kULNoSearchType=0,
    kULBrowsingSearch,
    kULStemmedSearch,
    kULRegexSearch,
    kULTempBrowsingSearch // for web versions
};


//////////////////////////////////////////////////////////////////
// Output types (for ULTranslator)
//////////////////////////////////////////////////////////////////
enum ULOutputType
{
    kULNoOutputType = -1,
    kULPrettyOutput = 0,
    kULRawOutput,
    kULTextOutput,
    kULWebOutput,
    kULCollinsWebOutput,
    kULXmlOutput,
    kULOldXHTMLOutput,
    kULXHTMLOutput
};


//////////////////////////////////////////////////////////////////
// Part-of-speech ranks
//////////////////////////////////////////////////////////////////
enum
{
    kULFirstPOSPRank = 1,
    kULEmptyPOSPRank = kULFirstPOSPRank, // The empty part of speech

    kULPronounRank=2, // Articles and pronouns
    kULNounRank=3, // Nouns and numbers       
    kULAdjectiveRank=4, // Adjectives and participles
    kULAdverbRank=5, // Adverbs, abbreviations, prefixes, suffixes, conjunctions, and prepositions
    kULVerbRank=6, // Verbs
    kULExpressionRank=7, // Expressions, interjections, proverbs, etc.

    kULMaxPOSPRank
};



//////////////////////////////////////////////////////////////////
// Debugging aids
//////////////////////////////////////////////////////////////////
#ifdef UL_TRACE
#define    ULTrace( x )    ULUtilities::Trace( ULUtilities::Pretrace x, __FILE__, __LINE__ ) 
#else
#define ULTrace(x)
#endif

#ifdef _DEBUG
#undef DEBUG
#define DEBUG
#endif

#ifdef NDEBUG
#undef DEBUG
#endif

#ifdef DEBUG

#ifdef UL_EXTERNAL_ASSERT
void ULExternalAssert(char *condition, char *file, int line);

#define        UL_ASSERT(a)    if( !(a) ) ULExternalAssert(#a, __FILE__, __LINE__)

#elif defined(UL_ANSI)
#ifdef UL_NOSTL
#define     UL_ASSERT(a)
#else
#define     UL_ASSERT(a)   do { if(!(a)) { cerr << __FILE__ << ":" << __LINE__ << " assertion failed '" << #a << "'" << endl; } } while(0)
#endif

#elif defined(UL_PALMOS)
#define        UL_ASSERT(a)    do { if( !(a) ){ char str[maxStrIToALen]; FrmCustomAlert( 1600, "assertion: ", __FILE__ ", ", StrIToA(str,__LINE__) ); } } while(0)

#else
#define     UL_ASSERT(a)
#endif // UL_EXTERNAL_ASSERT


#else // !DEBUG

#if defined( UL_SYMBIAN ) && defined( UL_EXTERNAL_ASSERT )
// For low memory tests, added UL_ASSERT in symbain release builds
void ULULAPIError();
#define        UL_ASSERT(a)    if( !(a) ) ULULAPIError()
#else
#define        UL_ASSERT(a)
#endif // UL_SYMBIAN

#ifndef     assert
#define     assert(a)
#endif

#endif // DEBUG

#ifdef UL_ENABLE_TIMING
extern uint32 t1, t2, t3, t4, t5, t6, t7, t8;

#ifdef UL_WINDOWSMOBILE
#define START_TIMING(a) uint32 base = GetTickCount()
#define STOP_TIMING(a)  a += GetTickCount() - base
#define START_TIMING1(a)  uint32 base1 = GetTickCount()
#define STOP_TIMING1(a)  a += GetTickCount() - base1

#elif defined(UL_PALMOS)
#define START_TIMING(a) uint32 tbase##a=TimGetTicks()
#define STOP_TIMING(a) a += TimGetTicks() - tbase##a
#define START_TIMING1(a) uint32 tbase1=TimGetTicks()
#define STOP_TIMING1(a) a += TimGetTicks() - tbase1

#else
#define START_TIMING(a)
#define STOP_TIMING(a)
#define START_TIMING1(a)
#define STOP_TIMING1(a)
#endif

#else
#define START_TIMING(a)
#define STOP_TIMING(a)
#define START_TIMING1(a)
#define STOP_TIMING1(a)

#endif

#endif // ULTYPES_H
