//
//  Ultralingua.js
//  Ultralingua
//
//  Created by Ben Kazez on 3/24/08.
//  Copyright 2008 Ultralingua. All rights reserved.
//

var expanded = false;
var collapseAnimation, collapseAnimator;
var dictionaryLanguages, indexOfSelectedDictionaryLanguage;

var scrollArea;
var scrollBar;
var doneButton;
var infoButton;

var ULContentTopCollapsed = -193;

var ULHeightCollapsed = 50;
var ULHeightExpanded = 211;
var ULWidth = 369;

var JSKeyCodeEscape = 27;
var JSKeyCodeEnter = 13;


function setup()
{
    if (!ULWidgetPlugin) {
        alert("Unable to load ULWidgetPlugin");
        displayError(getLocalizedString("The widget plugin could not be loaded. Please try re-installing the widget."));
        return false;
    }

    scrollBar = new AppleVerticalScrollbar($("scrollbar"));
    scrollArea = new AppleScrollArea($("scrollArea"), scrollBar);
    
    // Set collapsed/expanded state.
    expanded = getPreferenceForKey("expanded");
    if (expanded == null) expanded = false;

    // Until the expand-based resizeTo code works, we do this like so.
    window.resizeTo(ULWidth, ULHeightExpanded);
    collapseAnimationHandler(null, expanded ? 0 : ULContentTopCollapsed, 0, 0);
    
    // Load dictionary languages.
    dictionaryLanguages = ULWidgetPlugin.dictionaryLanguageTitles();
    if (dictionaryLanguages.length == 0) {
        displayError(getLocalizedString("No dictionaries could be found. Please ensure that Ultralingua 7 is installed."));
    } else {
        languagePopUpHTML = "";
        for (i = 0; i < dictionaryLanguages.length; i++) {
            if (dictionaryLanguages[i]) {
                languagePopUpHTML += "<option value=\"" + i + "\">" + dictionaryLanguages[i] + "</options>";
            }
        }
        $("languagePopUp").innerHTML = languagePopUpHTML;
    }
    
    // Select first available language for now.
    if ($("languagePopUp").options.length > 0) {
        $("languagePopUp").options[0].selected = true;
    }
    $("languagePopUpUI").innerHTML = $("languagePopUp").options[0].innerHTML;
    ULWidgetPlugin.setSelectedDictionaryLanguageIndex($("languagePopUp").options[0].value);
    
    doneButton = new AppleGlassButton($("doneButton"), "Done", hidePrefs);
    infoButton = new AppleInfoButton($("infoButton"), $("front"), "black", "black", showPrefs);
    
    // Strings.
    $("footer").innerHTML = '» <a href="javascript:openApp();">' + getLocalizedString("Open Ultralingua") + '</a>';
    $("ulTitleText").innerHTML = "Ultralingua";
    $("openAppText").innerHTML = '<a href="javascript:openApp();">' + getLocalizedString("Open Ultralingua") + '</a>';
}

function displayError(str)
{
    if (!expanded) {
        toggleExpanded();
    }
    
    $("error").innerHTML = str;
    $("result").innerHTML = "";
    $("error").style.display = "block";
}

function didYouMeanClicked(s)
{
    $("search").value = s;
    performSearch();
}

function performSearch()
{
    if (dictionaryLanguages.length == 0) return;
    
    str = $("search").value;
    if (str == "") {
        $("result").innerHTML = "";
        if (expanded) {
            setTimeout("toggleExpanded();", 0);
        }
    } else {
        if (expanded) {
            result.innerHTML = ULWidgetPlugin.performSearch(str);
        }
    }
    
    scrollArea.verticalScrollTo(0);
    scrollArea.refresh();
    $("search").focus();
}

function searchKeyPress(event)
{
    if (event.keyCode == JSKeyCodeEscape) {
        if (expanded) {
            $("result").innerHTML = "";
            $("search").value = "";
            setTimeout("toggleExpanded();", 0);
        }
    } else if (event.keyCode == JSKeyCodeEnter) {
        if (!expanded) {
            setTimeout("toggleExpanded();", 0);
        }
    }
}

function languagePopUpChanged(elem)
{
    $("languagePopUpUI").innerHTML = $("languagePopUp").options[$("languagePopUp").selectedIndex].innerHTML;

    var selectedLanguageIndex = $("languagePopUp").options[$("languagePopUp").selectedIndex].value;
    ULWidgetPlugin.setSelectedDictionaryLanguageIndex(selectedLanguageIndex);
    setPreferenceForKey(selectedLanguageIndex, "selectedLanguageIndex");
    
    performSearch();
}

// Collapsing/expanding.

function toggleExpanded()
{
    var start, end;
    if (expanded) {
        start = 0;
        end = ULContentTopCollapsed;
    } else {
        start = ULContentTopCollapsed;
        end = 0;
    }
    
    collapseAnimator = new AppleAnimator(500, 13);
    collapseAnimation = new AppleAnimation(start, end, collapseAnimationHandler);
    collapseAnimator.addAnimation(collapseAnimation);
    collapseAnimator.oncomplete = collapseAnimationComplete()
    collapseAnimator.start();
    
    setPreferenceForKey("expanded", expanded);
}

function collapseAnimationHandler(animation, current, start, finish)
{
    $("content").style.top = current + "px";
}

function collapseAnimationComplete()
{
    expanded = !expanded;
    performSearch();
}    

// Preferences.

function createKey(key) {
    if (window.widget) {
        key = widget.identifier + "-" + key;
    }
    return key;
}

function setPreferenceForKey(value, key) {
    widget.setPreferenceForKey(value, createKey(key));
}

function getPreferenceForKey(key) {
    var result;
    result = widget.preferenceForKey(createKey(key));
    if (!result) {
        result = widget.preferenceForKey(key);
    }

    return result;
}

function openApp()
{
    widget.openApplication("com.ultralingua.Ultralingua");
}

function showPrefs()
{
    var front = $("front");
    var back = $("back");
    
    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }
    
    front.style.display = "none";
    back.style.display = "block";
    
    if (window.widget) {
        setTimeout("widget.performTransition();", 0);
    }
}

function hidePrefs()
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");
    
    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }
    
    back.style.display = "none";
    front.style.display = "block";
    
    if (window.widget)
        setTimeout("widget.performTransition();", 0);
}

// Utilities.

function getLocalizedString(key)
{
    try {
        var ret = localizedStrings[key];
        if (ret === undefined) {
            ret = key;
        }
    } catch(ex) {
        ret = key;
    }
    
    return ret;
}

function $(id)
{
    return document.getElementById(id);
}

function onhide()
{
}

function onshow()
{
    var searchField = $("search");
    searchField.focus();
    searchField.select();
}

function onremove()
{
    setPreferenceForKey("expanded", null);
}

widget.onremove = onremove;
widget.onhide = onhide;
widget.onshow = onshow;
