/**
 * @file ulrendereroptions.h
 * @brief The interface for the <code>ULRendererOptions</code> class.
 * @author Jeff Ondich
 */

#ifndef ULRENDEREROPTIONS_H
#define ULRENDEREROPTIONS_H

#include "ulstring.h"

// Approximate maxima -- the actual number of matching terms varies so that a
// term's full expressions are always included in the rendered text.
const uint32 kULDefaultMaxMatchingTerms = 80;
const uint32 kULDefaultMaxStemmedTerms = 5;

enum {
    kULRendererMatchModeExact,
    kULRendererMatchModePrefix,
    kULRendererMatchModeBrowse
};

/**
 * @class ULRendererOptions
 * @brief A class used to contain the options controlling the
 * rendering of ULD data to XHTML or some other string form.
 * 
 * More later.
 */

class ULRendererOptions
{
public:
    ULRendererOptions();
    ULRendererOptions( const ULRendererOptions& options );
    virtual ~ULRendererOptions();

    virtual void clear();
    virtual ULRendererOptions& operator=( const ULRendererOptions& options );
    
    const ulstring& getWordOnClick() const;
    void setWordOnClick( const ulstring& s );
    const ulstring& getUserAdditionEditOnClick() const;
    void setUserAdditionEditOnClick( const ulstring& s );
    const ulstring& getUserAdditionDeleteOnClick() const;
    void setUserAdditionDeleteOnClick( const ulstring& s );
    const ulstring& getTermID() const;
    void setTermID( const ulstring& s );
    uint32 getMaxMatchingTerms() const;
    void setMaxMatchingTerms( uint32 n );
    uint32 getMaxStemmedTerms() const;
    void setMaxStemmedTerms( uint32 n );
    uint32 getMatchMode() const;
    void setMatchMode( uint32 n );
    
    bool hasAddWordMarks() const;
    void setAddWordMarks( bool b );
    bool hasAddTermIDs() const;
    void setAddTermIDs( bool b );
    bool hasUseRegexDisplay() const;
    void setUseRegexDisplay( bool b );
    bool hasEnableTTS() const;
    void setEnableTTS( bool b );
    bool hasShowExpressions() const;
    void setShowExpressions( bool b );
    bool hasStemSearchTerm() const;
    void setStemSearchTerm( bool b );
    const ulstring& getTargetPOSP() const; // If non-empty, getDictionaryXHTML will not display terms till reaching one with this posp.
    void setTargetPOSP( ulstring s );
    bool hasForceWordTags() const;
    void setForceWordTags( bool b );
    bool hasIPadMode() const;
    void setIPadMode( bool b );
    bool hasSuppressVerbTables() const;
    void setSuppressVerbTables( bool b );

private:
    ulstring mWordOnClick;
    ulstring mUserAdditionEditOnClick;
    ulstring mUserAdditionDeleteOnClick;
    ulstring mTermID;
    uint32 mMaxMatchingTerms;
    uint32 mMaxStemmedTerms;
    bool mAddWordMarks;
    bool mAddTermIDs;
    bool mUseRegexDisplay;
    uint32 mMatchMode;
    bool mEnableTTS;
    bool mShowExpressions;
    bool mStemSearchTerm;
    ulstring mTargetPOSP;
    bool mForceWordTags;
	bool mIPadMode;
    bool mSuppressVerbTables;
};

#endif

