#ifndef ULSTEMS_H
#define ULSTEMS_H

#include "ulstring.h"
#include "ulcontainers.h"

// The Tense constants are a subset of the Tag constants, but
// it is often more intuitive to think of a tense being passed
// to certain functions, so we'll define the ULTense type here.
// Similar statement about ULPerson, ULGender, and ULForm.
typedef int ULTense;
typedef int ULPerson;
typedef int ULGender;
typedef int ULForm;


class ULVerb
{
private:
    uint32                              mClassID;
    ulstring                            mInfinitive;

    // Kept sorted in increasing order by attribute type (the uint32).
    ULList< ULPair<uint32,ulstring> >   mAttributeList;

public:
                    ULVerb();
                    ULVerb( const ULVerb& v );
                    ~ULVerb();

    ULVerb&         operator=( const ULVerb& v );

    void            clear();

    uint32          getClassID() const { return mClassID; }         
    void            setClassID( uint32 id ) { mClassID = id; }
    void            getDisplayInfinitive( ulstring& s ) const;
    void            getInfinitive( ulstring& s ) const { s = mInfinitive; }
    const ulstring& getInfinitive() const { return mInfinitive; }
    void            setInfinitive( const ulstring& s ) { mInfinitive = s; }
    bool            getAttribute( uint32 type, ulstring& value ) const;
    bool            hasAttribute( uint32 type ) const;
    bool            addAttribute( uint32 type, const ulstring& value );

    static int      compare( const ulstring& a, const ulstring& b, bool ignoreCase, bool ignoreAccents );
    bool            operator==( const ULVerb& v ) const;
    bool            operator<( const ULVerb& v ) const;

    uint32          size( uint32 bytesForClassID=1 ) const;
    uint8           *write( uint8 *buffer, uint32 bytesForClassID=1 ) const;

#ifdef UL_ANSI
    void            write( ostream& out ) const;
#endif
};


enum
{
    kULNoStemType = 0,
    kULVerbStemType,
    kULOtherStemType,

    kULNStemTypes
};

typedef int ULStemType;

struct UL_API ULStem
{
    ULStemType      mType;

    ulstring        mOriginalString;
    ulstring        mStemmedString;
    ulstring        mPartOfSpeech;
    uint32          mPartOfSpeechRank;

    ULTense         mTense;
    ULPerson        mPerson;

    ULForm          mFormType;
    ULGender        mGender;

                    ULStem();
    void            clear();

    bool            operator<( const ULStem& b ) const;
    bool            operator==( const ULStem& b ) const;

    uint32          verbFormSize() const;
    uint8           *writeVerbForm( uint8 * buffer ) const;
};


enum
{
    kULCFirstMatchOnly = 0,
    kULCFirstMatchPerVerb,
    kULCAllMatches
};


#endif

