///////////////////////////////////////////////////////////////////
// ulconjugator.h
///////////////////////////////////////////////////////////////////

#ifndef ULCONJUGATOR_H
#define ULCONJUGATOR_H

#include "ulstems.h"
#include "ulmodules.h"
#include "ulsegmentedfile.h"

#ifdef UL_WINDOWSMOBILE
#include <windows.h>
#endif

// Class ID for unclassified verbs.  This should normally be used just for development
// of the databases, for verbs that haven't yet been covered.
const uint32 kULCUnclassifiedClass = 255;

// Class ID for verbs that have no verb model, but do have appropriate
// conjugation information on the verb line in the original data.
const uint32 kULCIrregularClass = 254;

// Shared tags.  (I call them "shared" even though some of them are specific
// to one language.  They are shared in the sense that the language-specific
// tense constants start after all of these shared constants.)

enum
{
    kULCNullTag = 0,

    kULCFirstStemmingTag,
        kULCStemmingRoot = kULCFirstStemmingTag,
        kULCStemmingPartOfSpeech,
        kULCStemmingFirstForm,
            kULCStemmingForm = kULCStemmingFirstForm,
            kULCStemmingNominativeSingular,
            kULCStemmingNominativePlural,
            kULCStemmingGenitiveSingular,
            kULCStemmingGenitivePlural,
            kULCStemmingDativeSingular,
            kULCStemmingDativePlural,
            kULCStemmingAccusativeSingular,
            kULCStemmingAccusativePlural,
        kULCStemmingLastForm = kULCStemmingAccusativePlural,
        kULCStemmingFirstGender,
            kULCStemmingNoGender = kULCStemmingFirstGender,
            kULCStemmingGenderNeuter,
            kULCStemmingGenderFeminine,
            kULCStemmingGenderMasculine,
        kULCStemmingLastGender = kULCStemmingGenderMasculine,
    kULCLastStemmingTag = kULCStemmingGenderMasculine,

    kULCFirstSoloTag, // 16 : 0x10
        kULCTransform = kULCFirstSoloTag, // 16 : 0x10
        kULCDefective,
        kULCAspirated,
        kULCImpersonal,
        kULCStrong,
        kULCTransitive,
        kULCIntransitive,
        kULCTransitiveIntransitive,
        kULCPronominal,
        kULCPronominalPossible,
        kULCStandalone,
        kULCInfinitiveEnding,
        kULCPrepositions,
        kULCOnly3pForms,
        kULCAuxSein,
        kULCAuxSeinHaben,
        kULCAuxEssere, // 32 : 0x20
        kULCAuxAvereEssere,
        kULCAuxEtre,
        kULCAuxEtreAvoir,
    kULCLastSoloTag = kULCAuxEtreAvoir,

    kULCFirstPairedTag,
        kULCClass = kULCFirstPairedTag,

        kULCFirstPerson,
            kULC1p = kULCFirstPerson,
            kULC2p,
            kULC3p,
            kULC4p,
            kULC5p,
            kULC6p,
            kULC7p,
        kULCLastPerson = kULC7p,

        kULCDef,
        kULCDefect,
        kULCPrefix2,
        kULCPrefix3,
        kULCPreteritStem, // 48 : 0x30
        kULCPresentSubjunctiveStem,
        kULCSubTwoStem,
        kULCPreteritStemFromList,
        kULCSubTwoStemFromList,
        kULCAltForm2p,

        kULCA1,
        kULCA2,
        kULCA3,
        kULCA4,
        kULCA5,
        kULCA6,
        kULCB1,
        kULCB2,
        kULCB3, 
        kULCB4,
        kULCB5, // 64 : 0x40
        kULCB6,
        kULCB7,
        kULCC1,
        kULCC2,
        kULCC3,
        kULCC4,
        kULCC5,
    kULCLastPairedTag = kULCC5,

    kULCNoTense,
    kULCNSharedTags,

    kULCFirstTense = kULCNSharedTags // 73 : 0x49
};


const int kULCTenseInfinitive = kULCFirstTense; // 73 : 0x49
const int kULCTensePastParticiple = kULCFirstTense + 1;

// English tenses
enum
{
    kULCENFirstTense = kULCFirstTense, // 73 : 0x49
        kULCENTenseInfinitive = kULCENFirstTense, // 73 : 0x49
        kULCENTensePastParticiple,
        kULCENTensePresentParticiple,

        kULCENTensePresent,
        kULCENTensePresentProgressive,
        kULCENTenseFuture,
        kULCENTenseFutureProgressive,
        kULCENTenseFuturePerfect,
        kULCENTensePreterit,
        kULCENTensePastProgressive,
        kULCENTensePresentPerfect,
        kULCENTensePluperfect,
        kULCENTenseConditional,
        kULCENTensePastConditional,
        kULCENTenseImperative,
    kULCENLastTense = kULCENTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// French tenses
enum
{
    kULCFRFirstTense = kULCFirstTense,
        kULCFRTenseInfinitive = kULCFRFirstTense, // 73 : 0x49
        kULCFRTensePastParticiple,
        kULCFRTenseMasculineSingularPastParticiple,
        kULCFRTenseMasculinePluralPastParticiple,
        kULCFRTenseFeminineSingularPastParticiple,
        kULCFRTenseFemininePluralPastParticiple,
        kULCFRTensePresentParticiple,

        kULCFRTensePresent,
        kULCFRTenseFuture,
        kULCFRTenseFuturePerfect,
        kULCFRTenseImperfect,
        kULCFRTensePasseCompose,
        kULCFRTensePluperfect,
        kULCFRTensePreterit,
        kULCFRTensePasseAnterieur,
        kULCFRTenseConditional,
        kULCFRTensePastConditional,
        kULCFRTensePresentSubjunctive,
        kULCFRTensePastSubjunctive,
        kULCFRTenseImperfectSubjunctive,
        kULCFRTensePluperfectSubjunctive,
        kULCFRTenseImperative,
    kULCFRLastTense = kULCFRTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Spanish tenses
enum
{
    kULCSPFirstTense = kULCFirstTense,
        kULCSPTenseInfinitive = kULCSPFirstTense, // 73 : 0x49
        kULCSPTensePastParticiple,
        kULCSPTenseMasculineSingularPastParticiple,
        kULCSPTenseMasculinePluralPastParticiple,
        kULCSPTenseFeminineSingularPastParticiple,
        kULCSPTenseFemininePluralPastParticiple,
        kULCSPTenseGerund,

        kULCSPTensePresent,
        kULCSPTenseImperfect,
        kULCSPTensePreterito,
        kULCSPTenseFuture,
        kULCSPTenseConditional,
        kULCSPTensePreteritoPerfecto,
        kULCSPTensePluscuamperfect,
        kULCSPTensePreteritAnterior,
        kULCSPTenseFutureAnterior,
        kULCSPTenseConditionalPerfect,
        kULCSPTensePresentSubjunctive,
        kULCSPTensePastSubjunctive,
        kULCSPTenseImperfectSubjunctive1,
        kULCSPTenseImperfectSubjunctive2,
        kULCSPTensePluscuamperfectSubjunctive1,
        kULCSPTensePluscuamperfectSubjunctive2,
        kULCSPTenseFutureSubjunctive,
        kULCSPTenseAnteriorFutureSubjunctive,
        kULCSPTenseImperative,
    kULCSPLastTense = kULCSPTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// German tenses
enum
{
    kULCGEFirstTense = kULCFirstTense,
        kULCGETenseInfinitive = kULCGEFirstTense, // 73 : 0x49
        kULCGETensePastParticiple,
        kULCGETensePresentParticiple,

        kULCGETensePresent,
        kULCGETenseFuture1,
        kULCGETenseFuture2,
        kULCGETensePreterit,
        kULCGETensePerfect,
        kULCGETensePluperfect,
        kULCGETenseSubjunctive1,
        kULCGETenseSubjunctive2,
        kULCGETenseSubjunctive1Future2,
        kULCGETenseSubjunctive2Future1,
        kULCGETenseSubjunctive2Future2,
        kULCGETenseSubjunctive1Perfect,
        kULCGETenseSubjunctive2Perfect,
        kULCGETenseImperative,
    kULCGELastTense = kULCGETenseImperative,

    // Non-tense features found in the verb lines.
    kULCGEFirstNonTenseFeature,
        kULCGESeparatedInfinitive = kULCGEFirstNonTenseFeature,
    kULCGELastNonTenseFeature = kULCGESeparatedInfinitive,
    kULCGEFirstBooleanNonTenseFeature
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Italian tenses
enum
{
    kULCITFirstTense = kULCFirstTense,
        kULCITTenseInfinitive = kULCITFirstTense, // 73 : 0x49
        kULCITTensePastParticiple,
        kULCITTenseMasculineSingularPastParticiple,
        kULCITTenseMasculinePluralPastParticiple,
        kULCITTenseFeminineSingularPastParticiple,
        kULCITTenseFemininePluralPastParticiple,
        kULCITTenseGerund,
        kULCITTensePresentParticiple,

        kULCITTensePresent,
        kULCITTenseFuture,
        kULCITTenseFutureAnterior,
        kULCITTenseImperfect,
        kULCITTensePerfect,
        kULCITTensePreterit,
        kULCITTensePluperfect,
        kULCITTensePastAnterior,
        kULCITTenseConditional,
        kULCITTensePastConditional,
        kULCITTensePresentSubjunctive,
        kULCITTenseImperfectSubjunctive,
        kULCITTensePluperfectSubjunctive,
        kULCITTensePastAnteriorSubjunctive,
        kULCITTenseImperative,
    kULCITLastTense = kULCITTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Portuguese tenses
enum
{
    kULCPOFirstTense = kULCFirstTense,
        kULCPOTenseInfinitive = kULCPOFirstTense, // 73 : 0x49
        kULCPOTensePastParticiple,
        kULCPOTenseMasculineSingularPastParticiple,
        kULCPOTenseMasculinePluralPastParticiple,
        kULCPOTenseFeminineSingularPastParticiple,
        kULCPOTenseFemininePluralPastParticiple,
        kULCPOTenseGerund,

        kULCPOTensePresent,
        kULCPOTenseFuture,
        kULCPOTenseFuturePerfect,
        kULCPOTenseImperfect,
        kULCPOTensePreterit,
        kULCPOTensePerfect,
        kULCPOTensePluperfect,
        kULCPOTensePreteritPluperfect,
        kULCPOTenseConditional,
        kULCPOTensePastConditional,
        kULCPOTensePresentSubjunctive,
        kULCPOTenseFutureSubjunctive,
        kULCPOTenseFuturePerfectSubjunctive,
        kULCPOTensePastSubjunctive,
        kULCPOTensePluperfectSubjunctive,
        kULCPOTenseImperfectSubjunctive,
        kULCPOTensePresentPersonalInfinitive,
        kULCPOTensePersonalInfinitivePerfect,
        kULCPOTenseImperative,
    kULCPOLastTense = kULCPOTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Norwegian tenses
enum
{
    kULCNOFirstTense = kULCFirstTense,
        kULCNOTenseInfinitive = kULCNOFirstTense, // 73 : 0x49
        kULCNOTensePastParticiple,
        kULCNOTensePerfectParticiple,
        kULCNOTensePresentParticiple,

        kULCNOTensePresentIndicative,
        kULCNOTensePastIndicative,
        kULCNOTenseFutureIndicative,
        kULCNOTensePresentPerfect,
        kULCNOTensePastPerfect,
        kULCNOTenseFuturePerfect,
        kULCNOTensePresentConditional,
        kULCNOTensePerfectConditional,
        kULCNOTenseImperative,
    kULCNOLastTense = kULCNOTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Latin tenses
enum
{
    kULCLatinFirstTense = kULCFirstTense,
        kULCLatinTenseInfinitive = kULCLatinFirstTense, // 73 : 0x49
        kULCLatinTensePastParticiple,
        kULCLatinTensePresentParticiple,
        kULCLatinTenseGerund,
        kULCLatinTenseGerundive,

        kULCLatinTensePresentIndicative,
        kULCLatinTensePresentSubjunctive,
        kULCLatinTenseImperfectIndicative,
        kULCLatinTenseImperfectSubjunctive,
        kULCLatinTenseFutureIndicative,
        kULCLatinTensePerfectIndicative,
        kULCLatinTensePerfectSubjunctive,
        kULCLatinTensePluperfectIndicative,
        kULCLatinTensePluperfectSubjunctive,
        kULCLatinTenseFuturePerfectIndicative,
        kULCLatinTensePresentIndicativePassive,
        kULCLatinTensePresentIndicativeDeponent,
        kULCLatinTensePresentSubjunctivePassive,
        kULCLatinTensePresentSubjunctiveDeponent,
        kULCLatinTenseImperfectIndicativePassive,
        kULCLatinTenseImperfectIndicativeDeponent,
        kULCLatinTenseImperfectSubjunctivePassive,
        kULCLatinTenseImperfectSubjunctiveDeponent,
        kULCLatinTenseFutureIndicativePassive,
        kULCLatinTenseFutureIndicativeDeponent,
        kULCLatinTensePerfectIndicativePassive,
        kULCLatinTensePerfectIndicativeDeponent,
        kULCLatinTensePerfectSubjunctivePassive,
        kULCLatinTensePerfectSubjunctiveDeponent,
        kULCLatinTensePluperfectIndicativePassive,
        kULCLatinTensePluperfectIndicativeDeponent,
        kULCLatinTensePluperfectSubjunctivePassive,
        kULCLatinTensePluperfectSubjunctiveDeponent,
        kULCLatinTenseFuturePerfectIndicativePassive,
        kULCLatinTenseFuturePerfectIndicativeDeponent,
        kULCLatinTenseImperative,
        kULCLatinTenseImperativeII,
    kULCLatinLastTense = kULCLatinTenseImperativeII,

    // Non-tense features found in the verb lines.
    kULCLatinFirstNonTenseFeature,
        kULCLatinFirstPersonPresent = kULCLatinFirstNonTenseFeature,
        kULCLatinAdditional,
    kULCLatinLastNonTenseFeature = kULCLatinAdditional,
    kULCLatinFirstBooleanNonTenseFeature
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Russian class IDs (note that they start below kULCIrregular = 254).
const uint32 kULCRUPrefixClass = 253;
const uint32 kULCRUReflexiveClass = 252;
const uint32 kULCRUToBeClass = 1;

// Russian tenses

enum
{
    kULCRUFirstTense = kULCFirstTense,
        kULCRUTenseInfinitive = kULCRUFirstTense, // 73 : 0x49

        kULCRUTensePresent,
        kULCRUTensePast,
        kULCRUTenseFuture,
        kULCRUTenseConditional,
        kULCRUTenseImperative,
    kULCRULastTense = kULCRUTenseImperative,

    // These are non-tense features with both open and close tags, found on verb lines.
    kULCRUFirstNonTenseFeature,
        kULCRUPrefix = kULCRUFirstNonTenseFeature,
        kULCRUPartner,

    // These are solo tags (i.e. boolean features for a verb)
    kULCRUFirstBooleanNonTenseFeature,
        kULCRUImperfective = kULCRUFirstBooleanNonTenseFeature,
        kULCRUPerfective,
        kULCRUMultidirectional,
        kULCRUUnidirectional,
    kULCRULastNonTenseFeature = kULCRUUnidirectional
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Dutch tenses
enum
{
    kULCNLFirstTense = kULCFirstTense,
        kULCNLTenseInfinitive = kULCNLFirstTense, // 73 : 0x49
        kULCNLTensePresentParticiple1,
        kULCNLTensePresentParticiple23,
        kULCNLTensePresentParticiple2pb,
        kULCNLTensePresentParticiple46,
        kULCNLTensePastParticiple13,
        kULCNLTensePastParticiple2,
        kULCNLTensePastParticiple46,
        kULCNLTensePerfectParticiple,

        kULCNLTensePresent,
        kULCNLTensePast,
        kULCNLTenseFuture,
        kULCNLTenseConditional,
        kULCNLTensePresentPerfect,
        kULCNLTensePastPerfect,
        kULCNLTenseFuturePerfect,
        kULCNLTenseConditionalPerfect,
        kULCNLTenseImperative,
    kULCNLLastTense = kULCNLTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Polish tenses
enum
{
    kULCPLFirstTense = kULCFirstTense,
        kULCPLTenseInfinitive = kULCPLFirstTense, // 73 : 0x49

        kULCPLTensePresentImperfect,
        kULCPLTensePastImperfect,
        kULCPLTenseImperativeImperfect,
        kULCPLTenseConditionalImperfect,
        kULCPLTensePresentPerfective,
        kULCPLTensePastPerfective,
        kULCPLTenseImperativePerfective,
        kULCPLTenseConditionalPerfective,
        kULCPLTenseImperfectiveFutureMasculineSingular,
        kULCPLTenseImperfectiveFutureFeminineSingular,
        kULCPLTenseImperfectiveFutureMasculinePlural,
        kULCPLTenseImperfectiveFutureFemininePlural,
    kULCPLLastTense = kULCPLTenseImperfectiveFutureFemininePlural,

    // These are non-tense features with both open and close tags, found on verb lines.
    kULCPLFirstNonTenseFeature,
        kULCPLPresent = kULCPLFirstNonTenseFeature,
        kULCPLPast,
        kULCPLImperative,
        kULCPLConditional,

    // These are solo tags (i.e. boolean features for a verb)
    kULCPLFirstBooleanNonTenseFeature,
        kULCPLImperfective = kULCPLFirstBooleanNonTenseFeature,
        kULCPLPerfective,
        kULCPLMultidirectional,
        kULCPLUnidirectional,
    kULCPLLastNonTenseFeature = kULCPLUnidirectional
    // DO NOT LET THIS GET BIGGER THAN 255.
};


// Chinese tenses
enum
{
    kULCCNFirstTense = kULCFirstTense,
        kULCCNTenseInfinitive = kULCCNFirstTense, // 73 : 0x49
        kULCCNTensePastParticiple,

        kULCCNTensePresent,
        kULCCNTensePast,
        kULCCNTenseFuture,
    kULCCNLastTense = kULCCNTenseFuture
    // DO NOT LET THIS GET BIGGER THAN 255.
};

// Klingon tenses
enum
{
    kULCKLFirstTense = kULCFirstTense,
        kULCKLTenseInfinitive = kULCKLFirstTense, // 73 : 0x49

        kULCKLTenseNoObject,
        kULCKLTenseMeObject,
        kULCKLTenseYouObject,
        kULCKLTenseItObject,
        kULCKLTenseUsObject,
        kULCKLTenseYouPluralObject,
        kULCKLTenseThemObject,

        kULCKLTenseImperativeNoObject,
        kULCKLTenseImperativeMeObject,
        kULCKLTenseImperativeItObject,
        kULCKLTenseImperativeUsObject,
        kULCKLTenseImperativeThemObject,

        kULCKLTenseVerbSuffixes,
    kULCKLLastTense = kULCKLTenseVerbSuffixes
    // DO NOT LET THIS GET BIGGER THAN 255.
};

// Turkish tenses
enum
{
    kULCTRFirstTense = kULCFirstTense,
        kULCTRTenseInfinitive = kULCTRFirstTense, // 73 : 0x49

        kULCTRTenseImperative,
    kULCTRLastTense = kULCTRTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};

// Haitian tenses
enum
{
    kULCHTFirstTense = kULCFirstTense,
        kULCHTTenseInfinitive = kULCHTFirstTense, // 73 : 0x49

        kULCHTTenseImperative,
    kULCHTLastTense = kULCHTTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};

// Swedish tenses
enum
{
    kULCSVFirstTense = kULCFirstTense,
        kULCSVTenseInfinitive = kULCSVFirstTense, // 73 : 0x49
        kULCSVTensePresentParticiple,
        kULCSVTensePastParticiple,
        kULCSVTensePerfectParticiple,
        kULCSVTensePastCon,

        kULCSVTenseIndicativePresent,
        kULCSVTenseIndicativePast,
        kULCSVTenseIndicativeFuture,
        kULCSVTenseIndicativePerfect,
        kULCSVTenseIndicativePluperfect,
        kULCSVTenseIndicativeFuturePerfect,
        kULCSVTenseConjunctivePresent,
        kULCSVTenseConjunctivePast,
        kULCSVTenseConjunctivePerfect,
        kULCSVTenseConditionalPresent,
        kULCSVTenseConditionalPerfect,
        kULCSVTenseImperative,
    kULCSVLastTense = kULCSVTenseImperative
    // DO NOT LET THIS GET BIGGER THAN 255.
};

struct ULSuffixModel
{
    uint32              mClassID;
    ULTense             mTense;
    ULPerson            mPerson;
    ulstring            mReversedSuffix;

                        ULSuffixModel();
                        ULSuffixModel( const ULSuffixModel& model );
                        ~ULSuffixModel();
    
    void                clear();
    ULSuffixModel&      operator=( const ULSuffixModel& smodel );

    bool                operator==( const ULSuffixModel& smodel ) const;
    bool                operator<( const ULSuffixModel& smodel ) const;

    uint32              size( uint32 bytesForClassID=1 ) const;
    uint8               *write( uint8 *buffer, uint32 bytesForClassID=1 ) const;
};


struct ULStemmingModel
{
    ULForm              mFormType;
    ulstring            mReversedFormSuffix;
    ulstring            mPartOfSpeech;
    uint32              mPartOfSpeechRank;
    ULGender            mGender;
    ulstring            mRootSuffix;

                        ULStemmingModel();
                        ULStemmingModel( const ULStemmingModel& model );
                        ~ULStemmingModel();
    
    void                clear();
    ULStemmingModel&    operator=( const ULStemmingModel& model );

    bool                operator==( const ULStemmingModel& model ) const;
    bool                operator<( const ULStemmingModel& model ) const;

    uint32              size() const;
    uint8               *write( uint8 *buffer ) const;
};


struct ULTenseInfo
{
    ULTense                             mTense;
    ULList< ULPair<uint32,ulstring> >   mPersonList;

                    ULTenseInfo();
                    ULTenseInfo( const ULTenseInfo& td );
                    ~ULTenseInfo();

    ULTenseInfo&    operator=( const ULTenseInfo& td );

    void            clear();
};


class ULVerbModel
{
    friend class ULConjugator;

public:
    uint32                              mClassID;
    ULList< ULPair<uint32,ulstring> >   mAttributeList;
    ULList< ULTenseInfo >               mTenseList;

                    ULVerbModel();
                    ULVerbModel( const ULVerbModel& vm );
                    ~ULVerbModel();

    ULVerbModel&    operator=( const ULVerbModel& vm );

    void            clear();

    bool            addAttribute( uint32 type, const ulstring& value );
    bool            getAttribute( uint32 type, ulstring& value ) const;
    bool            hasAttribute( uint32 type ) const;
    bool            getSuffix( ULTense tense, ULPerson person, ulstring& suffix ) const;
    uint32          getTransformType( int n ) const;
    ULList<ULTenseInfo>& getTenseList();

    uint8           *write( uint8 *buffer, uint32 bytesForClassID=1  ) const;
    uint32          size( uint32 bytesForClassID=1 ) const;
};


enum
{
    kULCFirstSegment = 0,
    kULCHeaderSegment = kULCFirstSegment,
    kULCVerbClassIndexSegment,
    kULCPersonNameSegment,
    kULCTenseNameSegment,
    kULCVerbModelSegment,
    kULCVerbSegment,
    kULCStemmingModelSegment,
    kULCSuffixModelSegment,
    kULCIrregularVerbSegment,
    kULCIrregularPluralSegment,

    kULCNSegments
};


struct ULConjugationResult
{
    ulstring mInfinitive;
    ulstring mConjugatedForm;
    ULTense mTense;
    ulstring mTenseName;
    ULPerson mPerson;
    ulstring mPersonName;
};


// I like having my preambles take up a multiple of
// sixteen bytes so they display nicely in binary
// editors like the one in Visual Studio.  That's why
// there are reserved fields here.
struct ULCHeader
{
    // If you change the header, don't forget to change kULCHeaderSize.
    uint16          mMagicNumber;
    ULLanguage      mLanguage;
    uint32          mFileSize;
    uint32          mVersion;
    uint32          mNVerbClasses;
    uint32          mNSuffixModels;
    uint32          mNDissectionRules;
    uint32          mUnused2;
    uint32          mUnused3;

                    ULCHeader();
                    ~ULCHeader();

    void            clear();
    bool            checkIntegrity() const;
    uint8           *write( uint8 *buffer ) const;
};

const uint32 kULCHeaderSize = 32;
const uint32 kULCBlockSize = 0x400;

// See comment in ulcmaker.cpp, process_verb_model.
const uint32 kULCTransformOffset = 2;

// When we're stemming a verb form, like "parlions", we chop the form down to
// a prefix that matches some verb in our conjugator database.  In this case,
// we have to chop all the way down to "parl" before we find a match.  How
// far are we willing to chop?  This constant tells us.
#ifdef UL_PALMOS
const uint32 kULCMinRootLettersToConsider = 4;
#elif defined(UL_SYMBIAN)
const uint32 kULCMinRootLettersToConsider = 3;
#else
const uint32 kULCMinRootLettersToConsider = 2;
#endif

const uint32 kULCMaxVerbsToCheckAfterSuccess = 8;

#if defined(UL_SYMBIAN) || defined(UL_PALMOS)
const uint32 kULCMaxVerbsToCheck = 36;
#else
const uint32 kULCMaxVerbsToCheck = 100;//36;
#endif

struct ULSortHelper
{
    ULList<ulstring>    mStringList;
    bool                mIgnoreCase;
    bool                mIgnoreAccents;
};


class ULConjugator
{
    friend class ULVerb;

public:
    static ULConjugator *create();

                    ULConjugator() ;
                    ~ULConjugator() ;

    bool            init();

    enum { kULPalmRecordSize = 1024 };

    bool            attach( const ULModuleIdentifier& moduleID );
    void            detach();
    bool            open();
    bool            open( const ULModuleIdentifier& moduleID );
    void            close();
    bool            isOpen() const ;

    void            setCompressor( ULCompressor *compressor );

    ULLanguage      getLanguage() const { return mHeader.mLanguage; }
    uint32          getVersion() const { return mHeader.mVersion; }
    uint32          getSize() const { return mHeader.mFileSize; }
    const ULModuleIdentifier &getModuleIdentifier() const { return mModuleIdentifier; }

    bool            getVerb( const ulstring& searchString, ULVerb& verb ) ;
    void            getVerbs( const ulstring& searchString, ULList<ULVerb>& verbList );
    void            getMatchingVerbs( const ulstring& searchString,
                                      ULList<ulstring>& verbs,
                                      bool clearList=false,
                                      uint32 maxVerbCount=0 );
    void            getAllVerbs( ULList<ulstring>& verbs );

    bool            getConjugation( const ULVerb& verb, ULTense tense, ULPerson person, ulstring& conjugatedVerb ) ;
    bool            getConjugation( const ULVerb& verb, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList ) ;
    bool            getConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList ) ;
    void            getVerbDetails( const ULVerb& verb, ulstring& details ) ; 

    void            getTenses( const ULVerb& v, ULList<ULTense>& tenseList, bool includeParticiples=false ) ;
    void            getTensesForClass( const ULVerb& v, uint32 classID, ULList<ULTense>& tenseList, bool includeParticiples=false ) ;
    void            getAllTenses( ULList<ULTense>& tenseList, bool includeParticiples=false ) ;
    void            getTenseName( ULTense tense, ulstring& name ) ;
    void            getPersons( const ULVerb& v, ULTense tense, ULList<ULPerson>& personList ) ;
    void            getPersonsForClass( const ULVerb& v, uint32 classID, ULTense tense, ULList<ULPerson>& personList ) ;
    void            getPersonName( ULPerson person, ulstring& name, ULTense tense = kULCNoTense ) ;
    void            getEnglishPersonDescription( ULPerson person, ulstring& description ) ;
    void            getConjugationDisplayText( const ULVerb& verb, ulchar separator, ULTense tense, ULPerson person, ulstring& text ) ;

    static ULTense  getLastTense( ULLanguage lang ) ;
    static ULTense  getFirstBooleanNonTenseFeature( ULLanguage lang ) ;
    static ULTense  getLastNonTenseFeature( ULLanguage lang ) ;
    static ULTense  getPastParticipleTense( ULLanguage lang ) ;
    static bool     isCompoundTense( ULTense tense, ULLanguage lang ) ;
    static bool     isParticiple( ULTense tense, ULLanguage lang ) ;
    static void     splitCommaDelimitedString( const ulstring& str, ULList<ulstring>& stringList ) ;

    void            getVerbStems( const ulstring& s, ULList<ULStem>& candidateList, int strategy=kULCAllMatches ) ;
    void            getNonverbStems( const ulstring& s, ULList<ULStem>& candidateList ) ;

    bool            mReportProgress;
    void            setReportProgress( bool reportProgress );
    void            updateProgress() const;

    static void     removeSeparators( ulstring& s );
    static void     russianPrepareAccentsForDisplay( ulstring& text );

private:
    ULCHeader           mHeader;
    ULSegmentedFile     mSegmentedFile;
    ULModuleIdentifier  mModuleIdentifier;
    ULVerbModel         *mVerbModels;
    ULSuffixModel       *mSuffixModels;
    uint32              mNSuffixModels;

    uint8           getByte( uint32 offset ) ;
    uint32          getInt32( uint32 offset ) ;
    uint32          getString( uint32 offset, uint8 delimiter, ulstring& s ) ;

    void            loadVerbModels();
    ULVerbModel&    getVerbModel( uint32 classID );

    void            loadSuffixModels();

    void            cleanString( ulstring& s, bool ignoreCase=true, bool ignoreAccents=true );
    void            getTransformedSearchStrings( const ulstring& originalString, ULList<ulstring>& stringList ) ;

    static void     addToStemList( const ULStem& stem, ULList<ULStem>& stemList ) ;

    void            getSuffixModel( uint32 offset, ULSuffixModel& model ) ;
    bool            findSuffixModels( const ulstring& searchString, ULList<ULSuffixModel>& modelList ) ;

    void            getStemmingModel( uint32 offset, ULStemmingModel& model ) ;
    void            getAllStemmingModels( ULList<ULStemmingModel>& modelList ) ;
    bool            findStemmingModels( const ulstring& searchString, ULList<ULStemmingModel>& modelList ) ;

    void            getIrregularVerbForm( uint32 offset, ULStem& stem ) ;
    bool            findIrregularVerbForms( const ulstring& searchString, ULList<ULStem>& stemList ) ;

    void            getVerb( uint32 offset, ULVerb& verb ) ;
    bool            findVerb( const ulstring& searchString, uint32& offset, ULVerb& verb ) ;
    bool            getSuffixes( ULTense tense, ULPerson person, uint32 classID, const ULVerb& verb, ULList<ulstring>& suffixList );
    void            getSuffix( ULTense tense, ULPerson person, const ULVerb& verb, const ULVerbModel& vm, ulstring& suffix ) const;
    void            getTruncatedInfinitive( const ULVerb& verb, const ULVerbModel& vm, ULTense suffixFeature, ulstring& truncatedInfinitive );

    void            englishGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            frenchGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            spanishGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            spanishDoTransformation( const ulstring& suffix, uint32 transformationType, ULTense tense, ULPerson person, ulstring& conjugatedVerb );
    void            germanGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            germanDoTransformation( ulstring& suffix, uint32 transformationType, ULTense tense, ULPerson person, ulstring& v );
    void            italianGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            italianDoTransformation( const ulstring& suffix, uint32 transformationType, ULTense tense, ULPerson person, ulstring& v );
    void            portugueseGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            portugueseDoTransformation( const ulstring& suffix, uint32 transformationType, ULTense tense, ULPerson person, ulstring& v );
    void            latinGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            russianGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            russianDoTransformation( ulstring& suffix, uint32 transformationType, ULTense tense, ULPerson person, uint32 classId, ulstring& conjugatedVerb );
    void            dutchGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            polishGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            klingonGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            klingonGetVerbStems( const ulstring& s, ULList<ULStem>& candidateList, int strategy=kULCAllMatches ) ;
    void            klingonGetNonverbStems( const ulstring& s, ULList<ULStem>& candidateList ) ;
    void            norwegianGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );
    void            swedishGetConjugation( const ULVerb& verb, uint32 classID, ULTense tense, ULPerson person, ULList<ulstring>& conjugationList );

#if defined(UL_ANSI) && !defined(UL_NOSTL)
public:
    void            dumpAllVerbs( ostream& out );
    void            dumpAllConjugations( ostream& out, bool includeCompoundTenses );
#endif
};

    
//const uchar kULCEncryptor = uint8(0x00);
const uchar kULCEncryptor = uchar(0x79);

#endif
