////////////////////////////////////////////////////////////////////////////////
//
// ultokenizer.h
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ULTOKENIZER_H
#define ULTOKENIZER_H

#include "ulstring.h"
#include "ulcontainers.h"

struct ULTokenizerWord
{
    const char *mString;
    ULLanguage mLanguage;
};


struct UL_API ULTokenizerHashKey
{
    ulstring            mString;
    ULLanguage          mLanguage;

                        ULTokenizerHashKey();
                        ULTokenizerHashKey( const char *str, ULLanguage lang );
                        ULTokenizerHashKey( const ULTokenizerHashKey& hk );
                        ~ULTokenizerHashKey();

    ULTokenizerHashKey& operator=( const ULTokenizerHashKey& hk );
    bool                operator==( const ULTokenizerHashKey& hk ) const;
    bool                operator<( const ULTokenizerHashKey& hk ) const;

    uint32              hash( uint32 tableSize ) const;

};


class ULTokenizer
{
private:
    bool                mInitialized;
    ULHashTable<ULTokenizerHashKey,uint32>    mWords;

    enum
    {
        kULTokenizerTableSize = 401,
        kULTokenizerDefaultPartOfSpeechRank = 100
    };

public:
    enum { kULMinPartOfSpeechRank=1, kULMaxPartOfSpeechRank=6 };

    static ULTokenizer  *create();

                        ULTokenizer();
                        ~ULTokenizer();
    bool                init();
    bool                initialized() { return mInitialized; }

    uint32              getNextRawToken( ulstring_iterator& it, ulstring& token );

    void                getTokenList( ULList< ULPair<ulstring,uint32> >& tokenList,
                                      uint32& nRawTokens,
                                      ULLanguage lang,
                                      const ulstring& term );

    void                getTokenListWithCharIndices( ULList< ULPair<ulstring,uint32> >& tokenList,
                                                     uint32& nRawTokens,
                                                     ULLanguage lang,
                                                     const ulstring& term );
    
    bool                isCommonWord( const ulstring& s, ULLanguage lang );
};

#endif
